<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\AboutMeSection;
use App\Models\Translation;
use App\Models\TranslationKey;
use App\Rules\ImageValidationRule;
use App\Services\UploadedFilesService;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Illuminate\View\View;

class AboutMeSectionController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.about.index',
            'label' => 'Liste des sections',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.about.create',
            'label' => 'Créer une section',
            'icon' => 'plus',
        ],
    ];

    public function __construct(protected UploadedFilesService $uploadedFilesService) {}

    public function index(): View
    {
        $sections = AboutMeSection::with(['attractDescTransKey.translations', 'contentTransKey.translations'])->get();

        return view('admin.about.index', [
            'headerRoutes' => $this->headerRoutes,
            'sections' => $sections,
        ]);
    }

    public function createPage(): View
    {
        return view('admin.about.edit', [
            'title' => 'Créer une section',
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $lang = $request->input('lang', 'fr');
        $section = AboutMeSection::findOrFail($id);
        $attractDesc = $section->attractDescTransKey->getTranslation($lang);
        $content = $section->contentTransKey->getTranslation($lang);
        $active = false;

        if ($section->active == 1) {
            $active = true;
        }

        return view('admin.about.edit', [
            'title' => 'Modifier une section',
            'headerRoutes' => $this->headerRoutes,
            'section' => $section,
            'attractDesc' => $attractDesc,
            'content' => $content,
            'lang' => $lang,
            'active' => $active,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'attract_desc' => 'required|string',
            'content' => 'required|string',
            'picture' => [
                'required',
                new ImageValidationRule,
            ],
            'active' => 'sometimes|string|in:on',
        ]);

        $attractDescTransKey = TranslationKey::create(['key' => 'about_attract_desc_'.Str::uuid()]);
        $contentTransKey = TranslationKey::create(['key' => 'about_content_'.Str::uuid()]);

        Translation::create(['translation_key_id' => $attractDescTransKey->id, 'locale' => $request->input('lang'), 'text' => $request->input('attract_desc')]);
        Translation::create(['translation_key_id' => $contentTransKey->id, 'locale' => $request->input('lang'), 'text' => $request->input('content')]);

        try {
            $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('picture'));
        } catch (Exception $e) {
            Log::error('Error storing uploaded picture', ['exception' => $e]);

            return redirect()->route('admin.about.create')->withInput()->with('error', 'Erreur lors de l\'enregistrement de l\'image.');
        }

        if ($request->has('active')) {
            AboutMeSection::query()->update(['active' => false]);
        }

        AboutMeSection::create([
            'attract_desc_trans_key_id' => $attractDescTransKey->id,
            'content_trans_key_id' => $contentTransKey->id,
            'uploaded_picture_id' => $uploadedPicture->id,
            'active' => $request->has('active'),
        ]);

        return redirect()->route('admin.about.index')->with('success', 'Section créée avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'attract_desc' => 'required|string',
            'content' => 'required|string',
            'picture' => [
                'sometimes',
                'nullable',
                new ImageValidationRule,
            ],
            'active' => 'sometimes|string|in:on',
        ]);

        $section = AboutMeSection::findOrFail($id);

        Translation::updateOrCreate(
            ['translation_key_id' => $section->attractDescTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('attract_desc')]
        );

        Translation::updateOrCreate(
            ['translation_key_id' => $section->contentTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('content')]
        );

        if ($request->hasFile('picture')) {
            try {
                $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('picture'));
                $section->uploaded_picture_id = $uploadedPicture->id;
            } catch (Exception $e) {
                Log::error('Error storing uploaded picture', ['exception' => $e]);

                return redirect()->route('admin.about.edit', $id)->withInput()->with('error', 'Erreur lors de l\'enregistrement de l\'image.');
            }
        }

        if ($request->has('active')) {
            AboutMeSection::where('id', '!=', $section->id)->update(['active' => false]);
            $section->active = true;
        } else {
            $section->active = false;
        }

        $section->save();

        return redirect()->route('admin.about.edit', [
            'section' => $section->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Section mise à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $section = AboutMeSection::findOrFail($id);
        $section->delete();

        return redirect()->route('admin.about.index')->with('success', 'Section supprimée avec succès.');
    }
}
