<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Translation;
use App\Models\TranslationKey;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\View\View;

class CategoryController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.categories.index',
            'label' => 'Liste des catégories',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.categories.create',
            'label' => 'Créer une catégorie',
            'icon' => 'plus',
        ],
    ];

    public function index(): View
    {
        $categories = Category::with(['nameTranslationKey.translations' => function ($query) {
            $query->whereIn('locale', ['fr', 'en']);
        }])->get();

        return view('admin.categories.index', [
            'headerRoutes' => $this->headerRoutes,
            'categories' => $categories,
        ]);
    }

    public function createPage(): View
    {
        return view('admin.categories.edit', [
            'title' => 'Créer une catégorie',
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $category = Category::findOrFail($id);
        $name = $category->nameTranslationKey->getTranslation($request->input('lang', 'fr'));

        return view('admin.categories.edit', [
            'title' => 'Modifier une catégorie',
            'headerRoutes' => $this->headerRoutes,
            'category' => $category,
            'name' => $name,
            'lang' => $request->input('lang', 'fr'),
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'color' => ['required', 'string', 'in:'.implode(',', Category::COLORS)],
            'highlight' => 'accepted|sometimes',
        ]);

        $slug = Str::slug($request->input('slug'));

        if (Category::where('slug', $slug)->exists()) {
            return redirect()->route('admin.categories.create')
                ->withInput()
                ->with('error', 'Un catégorie avec ce slug existe déjà.');
        }

        $translationKey = TranslationKey::create([
            'key' => 'category_'.Str::uuid(),
        ]);

        Translation::create([
            'translation_key_id' => $translationKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('name'),
        ]);

        $category = Category::create([
            'slug' => $slug,
            'name_translation_key_id' => $translationKey->id,
            'color' => $request->input('color'),
            'highlight' => $request->has('highlight'),
        ]);

        return redirect()->route('admin.categories.edit', [
            'category' => $category->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Catégorie créée avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'color' => ['required', 'string', 'in:'.implode(',', Category::COLORS)],
            'highlight' => 'accepted|sometimes',
        ]);

        $slug = Str::slug($request->input('slug'));

        if (Category::where('slug', $slug)->where('id', '!=', $id)->exists()) {
            return redirect()->route('admin.categories.edit', ['category' => $id])
                ->withInput()
                ->with('error', 'Un catégorie avec ce slug existe déjà.');
        }

        $category = Category::findOrFail($id);
        $translationKey = $category->nameTranslationKey;

        Translation::updateOrCreate(
            [
                'translation_key_id' => $translationKey->id,
                'locale' => $request->input('lang'),
            ],
            [
                'text' => $request->input('name'),
            ]
        );

        $category->update([
            'slug' => $slug,
            'color' => $request->input('color'),
            'highlight' => $request->has('highlight'),
        ]);

        return redirect()->route('admin.categories.edit', [
            'category' => $category->id,
            'lang' => $request->input('lang')])
            ->with('success', 'Catégorie mise à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $category = Category::findOrFail($id);
        $category->delete();

        return redirect()->route('admin.categories.index')
            ->with('success', 'Catégorie supprimée avec succès.');
    }
}
