<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Event;
use App\Models\Translation;
use App\Models\TranslationKey;
use App\Rules\ImageValidationRule;
use App\Services\UploadedFilesService;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Illuminate\View\View;

class EventController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.events.index',
            'label' => 'Liste des évènements',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.events.create',
            'label' => 'Créer un évènement',
            'icon' => 'plus',
        ],
    ];

    public function __construct(protected UploadedFilesService $uploadedFilesService) {}

    public function index(): View
    {
        $events = Event::with('nameTranslationKey.translations')->get();

        return view('admin.events.index', [
            'headerRoutes' => $this->headerRoutes,
            'events' => $events,
        ]);
    }

    public function createPage(): View
    {
        return view('admin.events.edit', [
            'title' => 'Créer un évènement',
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $lang = $request->input('lang', 'fr');
        $event = Event::findOrFail($id);
        $name = $event->nameTranslationKey->translations->firstWhere('locale', $lang)->text ?? 'N/A';

        return view('admin.events.edit', [
            'title' => 'Modifier un évènement',
            'headerRoutes' => $this->headerRoutes,
            'event' => $event,
            'name' => $name,
            'lang' => $lang,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'date_start' => 'required|date',
            'date_end' => 'required|date|after_or_equal:date_start',
            'city' => 'required|string',
            'image' => [
                'required',
                new ImageValidationRule,
            ],
        ]);

        $slug = Str::slug($request->input('slug'));

        if (Event::where('slug', $slug)->exists()) {
            return redirect()->route('admin.events.create')
                ->withInput()
                ->with('error', 'Un évènement avec ce slug existe déjà.');
        }

        $nameTranslationKey = TranslationKey::create(['key' => 'event_'.Str::uuid()]);

        Translation::create([
            'translation_key_id' => $nameTranslationKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('name'),
        ]);

        try {
            $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('image'));
        } catch (Exception $e) {
            Log::error('Error while storing uploaded picture', [
                'exception' => $e,
            ]);

            return redirect()->route('admin.events.create')
                ->withInput()
                ->with('error', 'Une erreur est survenue lors de l\'enregistrement de l\'image.');
        }

        $event = Event::create([
            'slug' => $slug,
            'name_translation_key_id' => $nameTranslationKey->id,
            'date_start' => $request->input('date_start'),
            'date_end' => $request->input('date_end'),
            'city' => $request->input('city'),
            'uploaded_picture_id' => $uploadedPicture->id,
        ]);

        return redirect()->route('admin.events.edit', ['event' => $event->id])
            ->with('success', 'Évènement créé avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $event = Event::findOrFail($id);

        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'date_start' => 'required|date',
            'date_end' => 'required|date|after_or_equal:date_start',
            'city' => 'required|string',
            'image' => [
                'sometimes',
                'nullable',
                new ImageValidationRule,
            ],
        ]);

        $slug = Str::slug($request->input('slug'));

        if (Event::where('slug', $slug)->where('id', '!=', $id)->exists()) {
            return redirect()->route('admin.events.edit', ['event' => $id])
                ->withInput()
                ->with('error', 'Un évènement avec ce slug existe déjà.');
        }

        $nameTranslationKey = $event->nameTranslationKey;

        Translation::updateOrCreate(
            ['translation_key_id' => $nameTranslationKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('name')]
        );

        if ($request->hasFile('image')) {
            try {
                $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('image'));
            } catch (Exception $e) {
                Log::error('Error while storing uploaded picture', [
                    'exception' => $e,
                ]);

                return redirect()->route('admin.events.edit', [
                    'event' => $event->id,
                    'lang' => $request->input('lang'),
                ])
                    ->withInput()
                    ->with('error', 'Une erreur est survenue lors de l\'enregistrement de l\'image.');
            }

            $event->uploadedPicture()->associate($uploadedPicture);
        }

        $event->update([
            'slug' => $slug,
            'date_start' => $request->input('date_start'),
            'date_end' => $request->input('date_end'),
            'city' => $request->input('city'),
        ]);

        return redirect()->route('admin.events.edit', ['event' => $event->id])
            ->with('success', 'Évènement mis à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $event = Event::findOrFail($id);
        $event->delete();

        return redirect()->route('admin.events.index')
            ->with('success', 'Évènement supprimé avec succès.');
    }
}
