<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\LegalMention;
use App\Models\Translation;
use App\Models\TranslationKey;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\View\View;

class LegalMentionController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.legal_mentions.index',
            'label' => 'Liste des mentions légales',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.legal_mentions.create',
            'label' => 'Créer une mention légale',
            'icon' => 'plus',
        ],
    ];

    public function index(): View
    {
        $mentions = LegalMention::with('contentTransKey.translations')->get();

        return view('admin.legal_mentions.index', [
            'headerRoutes' => $this->headerRoutes,
            'mentions' => $mentions,
        ]);
    }

    public function createPage(): View
    {
        return view('admin.legal_mentions.edit', [
            'title' => 'Créer une mention légale',
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $lang = $request->input('lang', 'fr');
        $mention = LegalMention::findOrFail($id);
        $content = $mention->contentTransKey->getTranslation($lang);

        return view('admin.legal_mentions.edit', [
            'title' => 'Modifier une mention légale',
            'headerRoutes' => $this->headerRoutes,
            'mention' => $mention,
            'content' => $content,
            'lang' => $lang,
            'active' => $mention->active,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'content' => 'required|string',
            'active' => 'sometimes|string|in:on',
        ]);

        $contentTransKey = TranslationKey::create(['key' => 'legal_content_'.Str::uuid()]);

        Translation::create(['translation_key_id' => $contentTransKey->id, 'locale' => $request->input('lang'), 'text' => $request->input('content')]);

        $mention = LegalMention::create([
            'content_trans_key_id' => $contentTransKey->id,
            'active' => $request->has('active'),
        ]);

        $this->updateActiveStatus($mention, $request->has('active'));
        $mention->save();

        return redirect()->route('admin.legal_mentions.index')->with('success', 'Mention légale créée avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'content' => 'required|string',
            'active' => 'sometimes|string|in:on',
        ]);

        $mention = LegalMention::findOrFail($id);

        Translation::updateOrCreate(
            ['translation_key_id' => $mention->contentTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('content')]
        );

        $this->updateActiveStatus($mention, $request->has('active'));
        $mention->save();

        return redirect()->route('admin.legal_mentions.edit', [
            'id' => $mention->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Mention légale mise à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $mention = LegalMention::findOrFail($id);
        $mention->delete();

        return redirect()->route('admin.legal_mentions.index')->with('success', 'Mention légale supprimée avec succès.');
    }

    private function updateActiveStatus(LegalMention $mention, bool $active): void
    {
        if ($active) {
            LegalMention::where('id', '!=', $mention->id)->update(['active' => false]);
            $mention->active = true;
        } else {
            $mention->active = false;
        }
    }
}
