<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Prestation;
use App\Models\Translation;
use App\Models\TranslationKey;
use App\Rules\ImageValidationRule;
use App\Services\UploadedFilesService;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use Illuminate\View\View;

class PrestationController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.prestations.index',
            'label' => 'Liste des prestations',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.prestations.create',
            'label' => 'Créer une prestation',
            'icon' => 'plus',
        ],
    ];

    public function __construct(protected UploadedFilesService $uploadedFilesService) {}

    public function index(): View
    {
        $prestations = Prestation::with(['nameTransKey.translations', 'category'])->get();

        return view('admin.prestations.index', [
            'headerRoutes' => $this->headerRoutes,
            'prestations' => $prestations,
        ]);
    }

    public function createPage(): View
    {
        $categories = Category::with('nameTranslationKey.translations')
            ->get()
            ->mapWithKeys(function ($category) {
                $name = $category->nameTranslationKey->translations->firstWhere('locale', 'fr')->text ?? 'N/A';

                return [$category->id => $name];
            });

        return view('admin.prestations.edit', [
            'title' => 'Créer une prestation',
            'headerRoutes' => $this->headerRoutes,
            'categories' => $categories,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $lang = $request->input('lang', 'fr');
        $prestation = Prestation::findOrFail($id);
        $name = $prestation->nameTransKey->translations->firstWhere('locale', $lang)->text ?? 'N/A';
        $attractionDesc = $prestation->attractDescTransKey->translations->firstWhere('locale', $lang)->text ?? 'N/A';
        $contentDesc = $prestation->contentDescTransKey->translations->firstWhere('locale', $lang)->text ?? 'N/A';

        $categories = Category::with('nameTranslationKey.translations')
            ->get()
            ->mapWithKeys(function ($category) {
                $name = $category->nameTranslationKey->translations->firstWhere('locale', 'fr')->text ?? 'N/A';

                return [$category->id => $name];
            });

        return view('admin.prestations.edit', [
            'lang' => $request->input('lang', 'fr'),
            'title' => 'Modifier une prestation',
            'name' => $name,
            'attractionDesc' => $attractionDesc,
            'contentDesc' => $contentDesc,
            'headerRoutes' => $this->headerRoutes,
            'prestation' => $prestation,
            'categories' => $categories,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'category_id' => 'required|exists:categories,id',
            'icon_svg' => 'sometimes|file|mimes:svg',
            'attract_desc' => 'required|string',
            'content_desc' => 'required|string',
            'detail' => 'nullable|sometimes|string',
            'cover_image' => [
                'sometimes',
                new ImageValidationRule,
            ],
        ]);

        $iconSvgContent = '';

        $uploadedPicture = null;
        if ($request->has('cover_image')) {
            try {
                $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('cover_image'));
            } catch (Exception $e) {
                Log::error('Error while storing uploaded picture', [
                    'exception' => $e,
                ]);

                return redirect()->route('admin.prestations.create')
                    ->with('error', 'Erreur lors de l\'enregistrement de l\'image.');
            }
        }

        if ($request->has('icon_svg')) {
            $iconSvgContent = file_get_contents($request->file('icon_svg')->getRealPath());

            if (Str::contains($iconSvgContent, '<svg') === false) {
                return redirect()->back()->with('error', 'Le fichier fourni n\'est pas un fichier SVG valide.');
            }
        }

        $slug = Str::slug($request->input('slug'));
        $nameTransKey = TranslationKey::create(['key' => 'prestation_name_'.Str::uuid()]);
        Translation::create([
            'translation_key_id' => $nameTransKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('name'),
        ]);

        $attractDescTransKey = TranslationKey::create(['key' => 'prestation_attract_desc_'.Str::uuid()]);
        Translation::create([
            'translation_key_id' => $attractDescTransKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('attract_desc'),
        ]);

        $contentDescTransKey = TranslationKey::create(['key' => 'prestation_content_'.Str::uuid()]);
        Translation::create([
            'translation_key_id' => $contentDescTransKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('content_desc'),
        ]);

        $prestation = Prestation::create([
            'name_trans_key_id' => $nameTransKey->id,
            'slug' => $slug,
            'category_id' => $request->input('category_id'),
            'icon_svg' => $iconSvgContent,
            'attract_desc_trans_key_id' => $attractDescTransKey->id,
            'content_desc_trans_key_id' => $contentDescTransKey->id,
            'detail' => $request->input('detail', ''),
            'uploaded_picture_id' => $uploadedPicture->id ?? null,
        ]);

        return redirect()->route('admin.prestations.edit', [
            'prestation' => $prestation->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Prestation créée avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $prestation = Prestation::findOrFail($id);

        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'name' => 'required|string',
            'slug' => 'required|string',
            'category_id' => 'required|exists:categories,id',
            'icon_svg' => 'sometimes|file|mimes:svg',
            'attract_desc' => 'required|string',
            'content_desc' => 'required|string',
            'detail' => 'nullable|sometimes|string',
            'cover_image' => [
                'sometimes',
                new ImageValidationRule,
            ],
        ]);

        $uploadedPicture = null;
        if ($request->has('cover_image')) {
            try {
                $uploadedPicture = $this->uploadedFilesService->storeAndOptimizeUploadedPicture($request->file('cover_image'));
            } catch (Exception $e) {
                Log::error('Error while storing uploaded picture', [
                    'exception' => $e,
                ]);

                return redirect()->route('admin.prestations.edit', ['prestation' => $id])
                    ->with('error', 'Erreur lors de l\'enregistrement de l\'image.');
            }
        }

        if ($request->has('icon_svg')) {
            $iconSvgContent = file_get_contents($request->file('icon_svg')->getRealPath());

            if (Str::contains($iconSvgContent, '<svg') === false) {
                return redirect()->back()->with('error', 'Le fichier fourni n\'est pas un fichier SVG valide.');
            }

            $prestation->update(['icon_svg' => $iconSvgContent]);
        }

        $slug = Str::slug($request->input('slug'));
        $nameTransKey = $prestation->nameTransKey;
        Translation::updateOrCreate(
            ['translation_key_id' => $nameTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('name')]
        );

        $attractDescTransKey = $prestation->attractDescTransKey;
        Translation::updateOrCreate(
            ['translation_key_id' => $attractDescTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('attract_desc')]
        );

        $contentDescTransKey = $prestation->contentDescTransKey;
        Translation::updateOrCreate(
            ['translation_key_id' => $contentDescTransKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('content_desc')]
        );

        $prestation->update([
            'slug' => $slug,
            'category_id' => $request->input('category_id'),
            'detail' => $request->input('detail', ''),
            'uploaded_picture_id' => $uploadedPicture->id ?? $prestation->uploaded_picture_id,
        ]);

        return redirect()->route('admin.prestations.edit', [
            'prestation' => $prestation->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Prestation mise à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $prestation = Prestation::findOrFail($id);
        $prestation->delete();

        return redirect()->route('admin.prestations.index')
            ->with('success', 'Prestation supprimée avec succès.');
    }
}
