<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Mail\AdminInvitationMail;
use App\Models\DbConfig;
use App\Models\User;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use Illuminate\View\View;
use SlProjects\LaravelRequestLogger\app\Models\LoggedRequest;

class SettingsController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.settings.general',
            'label' => 'Général',
            'icon' => 'circle-user',
        ],
    ];

    protected bool $isSuperAdmin;

    public function __construct()
    {
        $this->isSuperAdmin = Auth::user()->email === config('app.protected_account_email');

        if ($this->isSuperAdmin) {
            $this->headerRoutes[] = [
                'route' => 'admin.settings.users',
                'label' => 'Utilisateurs',
                'icon' => 'users',
            ];
            $this->headerRoutes[] = [
                'route' => 'admin.settings.website-config',
                'label' => 'Configuration du site',
                'icon' => 'gear',
            ];
        }
    }

    public function userListPage(): View
    {
        $users = User::all();

        foreach ($users as $user) {
            $user->alreadyConnected = LoggedRequest::where('user_id', $user->id)->exists();
        }

        return view('admin.settings.users', [
            'headerRoutes' => $this->headerRoutes,
            'users' => $users,
            'isSuperAdmin' => $this->isSuperAdmin,
        ]);
    }

    public function createUser(Request $request): RedirectResponse
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
        ]);

        if (! $this->isSuperAdmin) {
            return redirect()->route('admin.settings.users')
                ->with('error', 'Vous n\'avez pas les droits pour effectuer cette action.');
        }

        // Generate a random password because it is meant to be reset by the user
        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make(Str::random()),
        ]);

        Mail::to($request->email)->send(new AdminInvitationMail($user));

        return redirect()->route('admin.settings.users')->with('success', 'Utilisateur créé.');
    }

    public function deleteUser(int $id): RedirectResponse
    {
        if (! $this->isSuperAdmin) {
            return redirect()->route('admin.settings.users')
                ->with('error', 'Vous n\'avez pas les droits pour effectuer cette action.');
        }

        $targetedUser = User::find($id);

        if ($targetedUser->email === config('app.protected_account_email')) {
            return redirect()->route('admin.settings.users')
                ->with('error', 'Vous ne pouvez pas supprimer ce compte.');
        }

        if ($targetedUser->id === auth()->id()) {
            return redirect()->route('admin.settings.users')
                ->with('error', 'Vous ne pouvez pas supprimer votre propre compte.');
        }

        $targetedUser->delete();

        return redirect()->route('admin.settings.users')->with('success', 'Utilisateur supprimé.');
    }

    public function generalPage(): View
    {
        return view('admin.settings.general', [
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function updateUserInfos(Request $request): RedirectResponse
    {
        $request->validate([
            'name' => 'sometimes|nullable|string|max:255',
            'email' => 'sometimes|nullable|string|email|max:255|unique:users',
            'password' => 'sometimes|nullable|string|min:8|confirmed',
        ]);

        $user = auth()->user();

        if ($request->filled('name')) {
            $user->name = $request->name;
        }

        if ($request->filled('email')) {
            if (User::where('email', $request->input('email'))->exists()) {
                return redirect()->route('admin.settings.general')
                    ->with('error', 'Cette adresse email est déjà utilisée.');
            }

            if (auth()->user()->email === config('app.protected_account_email')) {
                return redirect()->route('admin.settings.general')
                    ->with('error', "Il s'agit de l'address email protégée, vous ne pouvez pas la modifier.");
            }

            $user->email = $request->email;
        }

        if ($request->filled('password')) {
            $user->password = Hash::make($request->password);
        }

        $user->save();

        return redirect()->route('admin.settings.general')->with('success', 'Informations mises à jour.');
    }

    public function websiteConfigPage(): RedirectResponse|View
    {
        if (! $this->isSuperAdmin) {
            return redirect()->route('admin.settings.general')
                ->with('error', 'Vous n\'avez pas les droits pour effectuer cette action.');
        }

        return view('admin.settings.website-config', [
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function updateWebsiteConfig(Request $request): RedirectResponse
    {
        $request->validate([
            'maintenance_mode' => 'sometimes|nullable|string|in:true,false',
            'website_name' => 'sometimes|nullable|string|max:255',
        ]);

        if (! $this->isSuperAdmin) {
            return redirect()->route('admin.settings.general')
                ->with('error', 'Vous n\'avez pas les droits pour effectuer cette action.');
        }

        if ($request->has('maintenance_mode')) {
            DbConfig::set('maintenance_mode', $request->maintenance_mode);
        }

        if ($request->has('website_name')) {
            DbConfig::set('website_name', $request->website_name);
        }

        return redirect()->route('admin.settings.website-config')->with('success', 'Configuration mise à jour.');
    }
}
