<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\TermsSection;
use App\Models\Translation;
use App\Models\TranslationKey;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\View\View;

class TermsSectionController extends Controller
{
    protected array $headerRoutes = [
        [
            'route' => 'admin.terms.index',
            'label' => 'Liste des sections',
            'icon' => 'list',
        ],
        [
            'route' => 'admin.terms.create',
            'label' => 'Créer une section',
            'icon' => 'plus',
        ],
    ];

    public function index(): View
    {
        $sections = TermsSection::with(['contentTranslationKey.translations'])->get();

        return view('admin.terms.index', [
            'headerRoutes' => $this->headerRoutes,
            'sections' => $sections,
        ]);
    }

    public function createPage(): View
    {
        return view('admin.terms.edit', [
            'title' => 'Créer une section',
            'headerRoutes' => $this->headerRoutes,
        ]);
    }

    public function editPage(Request $request, int $id): View
    {
        $request->validate([
            'lang' => 'sometimes|string|in:fr,en',
        ]);

        $lang = $request->input('lang', 'fr');
        $section = TermsSection::findOrFail($id);
        $content = $section->contentTranslationKey->getTranslation($lang);
        $active = $section->active;

        return view('admin.terms.edit', [
            'title' => 'Modifier une section',
            'headerRoutes' => $this->headerRoutes,
            'section' => $section,
            'content' => $content,
            'lang' => $lang,
            'active' => $active,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'content' => 'required|string',
            'active' => 'sometimes|string|in:on',
        ]);

        $contentTranslationKey = TranslationKey::create(['key' => 'terms_content_'.Str::uuid()]);

        Translation::create([
            'translation_key_id' => $contentTranslationKey->id,
            'locale' => $request->input('lang'),
            'text' => $request->input('content'),
        ]);

        $section = TermsSection::create([
            'content_translation_key_id' => $contentTranslationKey->id,
            'active' => false,
        ]);

        $this->updateActiveStatus($section, $request->has('active'));
        $section->save();

        return redirect()->route('admin.terms.index')->with('success', 'Section créée avec succès.');
    }

    public function update(Request $request, int $id): RedirectResponse
    {
        $request->validate([
            'lang' => 'required|string|in:fr,en',
            'content' => 'required|string',
            'active' => 'sometimes|string|in:on',
        ]);

        $section = TermsSection::findOrFail($id);

        Translation::updateOrCreate(
            ['translation_key_id' => $section->contentTranslationKey->id, 'locale' => $request->input('lang')],
            ['text' => $request->input('content')]
        );

        $this->updateActiveStatus($section, $request->has('active'));
        $section->save();

        return redirect()->route('admin.terms.edit', [
            'section' => $section->id,
            'lang' => $request->input('lang'),
        ])->with('success', 'Section mise à jour avec succès.');
    }

    public function delete(int $id): RedirectResponse
    {
        $section = TermsSection::findOrFail($id);
        $section->delete();

        return redirect()->route('admin.terms.index')->with('success', 'Section supprimée avec succès.');
    }

    private function updateActiveStatus(TermsSection $section, bool $active): void
    {
        if ($active) {
            TermsSection::where('id', '!=', $section->id)->update(['active' => false]);
            $section->active = true;
        } else {
            $section->active = false;
        }
    }
}
