<?php

namespace App\Models;

use App\Services\ImageTranscodingService;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Intervention\Image\Drivers\Imagick\Driver;

class UploadedPicture extends Model
{
    use HasFactory;

    protected $fillable = [
        'path_original',
        'path_thumbnail',
        'path_medium',
        'path_big',
        'path_fullsize',
        'ratio',
        'height',
        'width',
    ];

    const THUMBNAIL_SIZE = 256;

    const MEDIUM_SIZE = 1024;

    const BIG_SIZE = 2048;

    public function optimize(): void
    {
        if (! Storage::disk('public')->exists($this->path_original)) {
            Log::warning('UploadedPicture optimization failed: file does not exist', [
                'path' => $this->path_original,
            ]);

            return;
        }
        $this->deleteOptimized();

        $originalImage = Storage::disk('public')->get($this->path_original);
        $imageTranscodingService = new ImageTranscodingService(new Driver);

        $dimensions = $imageTranscodingService->getDimensions($originalImage);
        $ratio = $this->getAspectRatio($dimensions['width'], $dimensions['height']);

        $thumbnailImage = $imageTranscodingService->transcode($originalImage, self::THUMBNAIL_SIZE);
        $mediumImage = $imageTranscodingService->transcode($originalImage, self::MEDIUM_SIZE);
        $bigImage = $imageTranscodingService->transcode($originalImage, self::BIG_SIZE);
        $fullsizeImage = $imageTranscodingService->transcode($originalImage);

        if (! $thumbnailImage || ! $mediumImage || ! $bigImage || ! $fullsizeImage) {
            Log::error('UploadedPicture optimization failed: transcoding failed', [
                'path' => $this->path_original,
            ]);

            return;
        }

        $thumbnailPath = Str::beforeLast($this->path_original, '.').'__thumbnail.avif';
        $mediumPath = Str::beforeLast($this->path_original, '.').'__medium.avif';
        $bigPath = Str::beforeLast($this->path_original, '.').'__big.avif';
        $fullsizePath = Str::beforeLast($this->path_original, '.').'__fullsize.avif';

        Storage::disk('public')->put($thumbnailPath, $thumbnailImage);
        Storage::disk('public')->put($mediumPath, $mediumImage);
        Storage::disk('public')->put($bigPath, $bigImage);
        Storage::disk('public')->put($fullsizePath, $fullsizeImage);

        // To the cdn
        if (config('app.cdn_disk')) {
            Storage::disk(config('app.cdn_disk'))->put($thumbnailPath, $thumbnailImage);
            Storage::disk(config('app.cdn_disk'))->put($mediumPath, $mediumImage);
            Storage::disk(config('app.cdn_disk'))->put($bigPath, $bigImage);
            Storage::disk(config('app.cdn_disk'))->put($fullsizePath, $fullsizeImage);
        }

        $this->path_thumbnail = $thumbnailPath;
        $this->path_medium = $mediumPath;
        $this->path_big = $bigPath;
        $this->path_fullsize = $fullsizePath;
        $this->ratio = $ratio;
        $this->width = $dimensions['width'];
        $this->height = $dimensions['height'];
        $this->save();
    }

    public function deleteOriginal(): void
    {
        if ($this->path_original && Storage::disk('public')->exists($this->path_original)) {
            Storage::disk('public')->delete($this->path_original);

            if (config('app.cdn_disk')) {
                Storage::disk(config('app.cdn_disk'))->delete($this->path_original);
            }
        }
    }

    public function deleteOptimized(): void
    {
        if ($this->path_thumbnail && Storage::disk('public')->exists($this->path_thumbnail)) {
            Storage::disk('public')->delete($this->path_thumbnail);

            if (config('app.cdn_disk')) {
                Storage::disk(config('app.cdn_disk'))->delete($this->path_thumbnail);
            }
        }

        if ($this->path_medium && Storage::disk('public')->exists($this->path_medium)) {
            Storage::disk('public')->delete($this->path_medium);

            if (config('app.cdn_disk')) {
                Storage::disk(config('app.cdn_disk'))->delete($this->path_medium);
            }
        }

        if ($this->path_big && Storage::disk('public')->exists($this->path_big)) {
            Storage::disk('public')->delete($this->path_big);

            if (config('app.cdn_disk')) {
                Storage::disk(config('app.cdn_disk'))->delete($this->path_big);
            }
        }

        if ($this->path_fullsize && Storage::disk('public')->exists($this->path_fullsize)) {
            Storage::disk('public')->delete($this->path_fullsize);

            if (config('app.cdn_disk')) {
                Storage::disk(config('app.cdn_disk'))->delete($this->path_fullsize);
            }
        }
    }

    public function delete(): void
    {
        $this->deleteOptimized();
        $this->deleteOriginal();
        parent::delete();
    }

    public function getOriginalSizeUrl(): string
    {
        if (config('app.cdn_disk')) {
            return Storage::disk(config('app.cdn_disk'))->url($this->path_original);
        }

        return Storage::disk('public')->url($this->path_original);
    }

    public function getThumbnailSizeUrl(): string
    {
        if (! $this->path_thumbnail || ! Storage::disk('public')->exists($this->path_thumbnail)) {
            return $this->getOriginalSizeUrl();
        }

        if (config('app.cdn_disk')) {
            return Storage::disk(config('app.cdn_disk'))->url($this->path_thumbnail);
        }

        return Storage::disk('public')->url($this->path_thumbnail);
    }

    public function getMediumSizeUrl(): string
    {
        if (! $this->path_medium || ! Storage::disk('public')->exists($this->path_medium)) {
            return $this->getOriginalSizeUrl();
        }

        if (config('app.cdn_disk')) {
            return Storage::disk(config('app.cdn_disk'))->url($this->path_medium);
        }

        return Storage::disk('public')->url($this->path_medium);
    }

    public function getBigSizeUrl(): string
    {
        if (! $this->path_big || ! Storage::disk('public')->exists($this->path_big)) {
            return $this->getOriginalSizeUrl();
        }

        if (config('app.cdn_disk')) {
            return Storage::disk(config('app.cdn_disk'))->url($this->path_big);
        }

        return Storage::disk('public')->url($this->path_big);
    }

    public function getFullsizeUrl(): string
    {
        if (! $this->path_fullsize || ! Storage::disk('public')->exists($this->path_fullsize)) {
            return $this->getOriginalSizeUrl();
        }

        if (config('app.cdn_disk')) {
            return Storage::disk(config('app.cdn_disk'))->url($this->path_fullsize);
        }

        return Storage::disk('public')->url($this->path_fullsize);
    }

    /**
     * Get the aspect ratio of an image
     *
     * @param  int  $width  The width of the image
     * @param  int  $height  The height of the image
     * @return string The aspect ratio of the image. Eg: 16:9
     */
    private function getAspectRatio(int $width, int $height): string
    {
        $greatestCommonDivisor = static function ($width, $height) use (&$greatestCommonDivisor) {
            return ($width % $height) ? $greatestCommonDivisor($height, $width % $height) : $height;
        };

        $divisor = $greatestCommonDivisor($width, $height);

        return $width / $divisor.':'.$height / $divisor;
    }
}
