<?php

namespace App\Rules;

use Closure;
use Illuminate\Contracts\Validation\ValidationRule;
use Illuminate\Http\UploadedFile;
use Imagick;

class ImageValidationRule implements ValidationRule
{
    protected ?int $maxWidth;

    protected ?int $maxHeight;

    protected int $maxSize; // In MB

    protected int $maxArea;

    public function __construct(?int $maxWidth = null, ?int $maxHeight = null, int $maxSize = 50, ?int $maxArea = null)
    {
        $this->maxWidth = $maxWidth ?? config('app.imagick.max_width');
        $this->maxHeight = $maxHeight ?? config('app.imagick.max_height');
        $this->maxSize = $maxSize;
        $this->maxArea = $maxArea ?? Imagick::getResourceLimit(Imagick::RESOURCETYPE_AREA);
    }

    public function validate(string $attribute, mixed $value, Closure $fail): void
    {
        if (! $value instanceof UploadedFile) {
            $fail("Le fichier fourni pour :attribute n'est pas valide.");

            return;
        }

        $supportedFormats = array_map(fn ($format) => 'image/'.$format, config('app.supported_image_formats'));
        if (! $value->isValid() || ! $value->isFile() || ! in_array($value->getMimeType(), $supportedFormats)) {
            $fail('Le fichier :attribute doit être une image valide.');

            return;
        }

        [$width, $height] = getimagesize($value->getRealPath());

        if ($this->maxWidth && $width > $this->maxWidth) {
            $fail("La largeur de l'image :attribute ne doit pas dépasser {$this->maxWidth} pixels.");
        }

        if ($this->maxHeight && $height > $this->maxHeight) {
            $fail("La hauteur de l'image :attribute ne doit pas dépasser {$this->maxHeight} pixels.");
        }

        $area = $width * $height;
        if ($this->maxArea && $area > $this->maxArea) {
            $fail("La surface totale de l'image :attribute ne doit pas dépasser {$this->maxArea} pixels (actuellement : $area pixels).");
        }

        $maxSizeInBytes = $this->maxSize * 1024 * 1024; // MB to Bytes
        if ($value->getSize() > $maxSizeInBytes) {
            $fail("La taille de l'image :attribute ne doit pas dépasser {$this->maxSize} MB.");
        }
    }
}
