<?php

namespace App\Services;

use App\Jobs\UploadedPictureJob;
use App\Models\UploadedPicture;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class UploadedFilesService
{
    public function __construct() {}

    public function pathCacheKey(string $path, ?string $disk = null): string
    {
        if (empty($disk)) {
            $disk = config('filesystems.default');
        }

        if (Str::endsWith($path, '/')) {
            $path = Str::beforeLast($path, '/');
        }

        return 'storage_folders.'.$disk.'.'.Str::replace('/', '-', $path);
    }

    /**
     * Refresh the file list cache for a given path, and return the list of files
     *
     * @param  string  $path  The path to refresh the cache for
     * @param  string|null  $disk  The disk to use. If null, the default disk will be used
     * @param  string|null  $cacheKey  The cache key to use. If null, a key will be generated
     * @return array The list of files in the path
     */
    public function refreshFileListCache(string $path, ?string $disk = null, ?string $cacheKey = null): array
    {
        if (empty($cacheKey)) {
            $cacheKey = $this->pathCacheKey($path, $disk);
        }

        if (empty($disk)) {
            $disk = config('filesystems.default');
        }

        Cache::forget($cacheKey);

        return Cache::rememberForever($cacheKey, function () use ($path, $disk) {
            return Collect(Storage::disk($disk)->files($path))->map(function ($path) {
                return pathinfo($path, PATHINFO_BASENAME);
            })->toArray();
        });
    }

    /**
     * Store an uploaded picture and optimize it
     *
     * @param  UploadedFile  $uploadedFile  The uploaded file
     * @return UploadedPicture The stored picture
     *
     * @throws Exception
     */
    public function storeAndOptimizeUploadedPicture(UploadedFile $uploadedFile): UploadedPicture
    {
        if (! in_array($uploadedFile->extension(), config('app.supported_image_formats'))) {
            throw new Exception('The specified file is not a picture or is not supported');
        }

        $folderName = 'uploads/'.Carbon::now()->format('Y/m/d');
        $fileName = $uploadedFile->hashName();

        $fileHasBeenSavedToLocal = $uploadedFile->store($folderName, 'public');

        if (! $fileHasBeenSavedToLocal) {
            $isFolderWritable = is_writable(Storage::disk('public')->path($folderName));

            throw new Exception('Error while saving the uploaded file. Is the folder writable? '.$isFolderWritable);
        }

        if (config('app.cdn_disk')) {
            $fileHasBeenSavedToCdn = $uploadedFile->store($folderName, config('app.cdn_disk'));

            if (! $fileHasBeenSavedToCdn) {
                throw new Exception('Error while saving the uploaded file to the CDN');
            }
        }

        $uploadedFileInstance = UploadedPicture::create([
            'path_original' => $folderName.'/'.$fileName,
            'ratio' => '',
        ]);

        UploadedPictureJob::dispatch($uploadedFileInstance);

        return $uploadedFileInstance;
    }
}
