<?php

namespace Tests\Feature\Controller;

use App\Http\Controllers\Admin\AboutMeSectionController;
use App\Models\AboutMeSection;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use PHPUnit\Framework\Attributes\CoversClass;
use Tests\TestCase;

#[CoversClass(AboutMeSectionController::class)] class AboutMeSectionTest extends TestCase
{
    use RefreshDatabase, WithoutMiddleware;

    public function test_it_can_list_about_me_sections()
    {
        AboutMeSection::factory()->count(3)->create();

        $response = $this->get(route('admin.about.index'));

        $response->assertStatus(200);
        $response->assertViewIs('admin.about.index');
        $response->assertViewHas('sections');
        $this->assertCount(3, $response->viewData('sections'));
    }

    public function test_it_can_create_an_about_me_section()
    {
        Storage::fake('public');
        $data = [
            'lang' => 'fr',
            'attract_desc' => 'Description d\'accroche',
            'content' => 'Contenu de la section',
            'picture' => UploadedFile::fake()->image('image.jpg'),
            'active' => 'on',
        ];

        $response = $this->post(route('admin.about.store'), $data);

        $response->assertRedirect(route('admin.about.index'));
        $this->assertDatabaseHas('translations', ['text' => 'Description d\'accroche', 'locale' => 'fr']);
        $this->assertDatabaseHas('translations', ['text' => 'Contenu de la section', 'locale' => 'fr']);
        $this->assertDatabaseHas('uploaded_pictures', ['id' => 1]);
        $this->assertDatabaseHas('about_me_sections', ['active' => true]);
    }

    public function test_it_fails_to_create_an_about_me_section_without_attract_desc()
    {
        Storage::fake('public');
        $response = $this->post(route('admin.about.store'), [
            'lang' => 'fr',
            'content' => 'Contenu de la section',
            'picture' => UploadedFile::fake()->image('image.jpg'),
        ]);

        $response->assertSessionHasErrors('attract_desc');
    }

    public function test_it_can_edit_an_about_me_section()
    {
        $section = AboutMeSection::factory()->create();

        $response = $this->get(route('admin.about.edit', ['section' => $section->id]));

        $response->assertStatus(200);
        $response->assertViewIs('admin.about.edit');
        $response->assertViewHas('section', $section);
    }

    public function test_it_can_update_an_about_me_section()
    {
        Storage::fake('public');
        $section = AboutMeSection::factory()->create();

        $data = [
            'lang' => 'fr',
            'attract_desc' => 'Nouvelle description d\'accroche',
            'content' => 'Nouveau contenu',
            'picture' => UploadedFile::fake()->image('new_image.jpg'),
        ];

        $response = $this->put(route('admin.about.update', ['section' => $section->id]), $data);

        $response->assertRedirect(route('admin.about.edit', ['section' => $section->id, 'lang' => 'fr']));
        $this->assertDatabaseHas('translations', ['text' => 'Nouvelle description d\'accroche', 'locale' => 'fr']);
        $this->assertDatabaseHas('translations', ['text' => 'Nouveau contenu', 'locale' => 'fr']);
        $this->assertDatabaseHas('about_me_sections', ['id' => $section->id, 'active' => 0]);
    }

    public function test_it_can_update_content_and_active_status()
    {
        AboutMeSection::factory()->count(4)->create();
        $section = AboutMeSection::factory()->create();

        $data = [
            'lang' => 'fr',
            'attract_desc' => 'Nouvelle description d\'accroche',
            'content' => 'Nouveau contenu',
            'active' => 'on',
        ];

        $response = $this->put(route('admin.about.update', ['section' => $section->id]), $data);

        $response->assertRedirect(route('admin.about.edit', ['section' => $section->id, 'lang' => 'fr']));
        $this->assertDatabaseHas('translations', ['text' => 'Nouvelle description d\'accroche', 'locale' => 'fr']);
        $this->assertDatabaseHas('translations', ['text' => 'Nouveau contenu', 'locale' => 'fr']);
        $this->assertDatabaseHas('about_me_sections', ['id' => $section->id, 'active' => 1]);

        // Assert that the other sections are not active
        $this->assertDatabaseHas('about_me_sections', ['id' => 1, 'active' => 0]);
        $this->assertDatabaseHas('about_me_sections', ['id' => 2, 'active' => 0]);
        $this->assertDatabaseHas('about_me_sections', ['id' => 3, 'active' => 0]);
        $this->assertDatabaseHas('about_me_sections', ['id' => 4, 'active' => 0]);
    }

    public function test_it_can_delete_an_about_me_section()
    {
        $section = AboutMeSection::factory()->create();

        $response = $this->get(route('admin.about.delete', ['section' => $section->id]));

        $response->assertRedirect(route('admin.about.index'));
        $this->assertDatabaseMissing('about_me_sections', ['id' => $section->id]);
    }
}
