<?php

namespace Controller;

use App\Http\Controllers\Admin\CategoryController;
use App\Models\Category;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\Test;
use Tests\TestCase;

#[CoversClass(CategoryController::class)] class CategoryTest extends TestCase
{
    use RefreshDatabase, WithoutMiddleware;

    #[Test]
    public function it_can_create_a_category_with_valid_name()
    {
        $response = $this->post('/admin/categories', [
            'name' => 'Nouvelle Catégorie',
            'slug' => 'nouvelle-categorie',
            'lang' => 'fr',
            'color' => 'blue',
            'highlight' => 'on',
        ]);

        $response->assertRedirectToRoute('admin.categories.edit', ['category' => 1, 'lang' => 'fr']);
        $this->assertDatabaseHas('categories', [
            'slug' => 'nouvelle-categorie',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Nouvelle Catégorie',
            'locale' => 'fr',
        ]);
    }

    #[Test]
    public function it_fails_to_create_a_category_without_name_and_color()
    {
        $response = $this->post('/admin/categories', [
            'slug' => 'nouvelle-categorie',
            'lang' => 'fr',
        ]);

        $response->assertSessionHasErrors('name');
    }

    #[Test]
    public function it_can_update_a_category_with_valid_name()
    {
        $category = Category::factory()->create();

        $response = $this->put("/admin/categories/$category->id", [
            'name' => 'Catégorie Modifiée',
            'slug' => 'categorie-modifiee',
            'lang' => 'fr',
            'color' => 'yellow',
            'highlight' => 'on',
        ]);

        $response->assertRedirectToRoute('admin.categories.edit', ['category' => $category->id, 'lang' => 'fr']);
        $this->assertDatabaseHas('categories', [
            'id' => $category->id,
            'slug' => 'categorie-modifiee',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Catégorie Modifiée',
            'locale' => 'fr',
        ]);
    }

    #[Test]
    public function it_fails_to_update_a_category_without_name()
    {
        $category = Category::factory()->create();

        $response = $this->put("/admin/categories/$category->id", [
            'slug' => 'categorie-modifiee',
            'lang' => 'fr',
        ]);

        $response->assertSessionHasErrors('name');
    }

    #[Test]
    public function it_transforms_slug_correctly()
    {
        $response = $this->post('/admin/categories', [
            'name' => 'Une Catégorie',
            'slug' => 'Une Catégorie avec des Espaces!',
            'lang' => 'fr',
            'color' => 'green',
        ]);

        $response->assertRedirectToRoute('admin.categories.edit', ['category' => 1, 'lang' => 'fr']);
        $this->assertDatabaseHas('categories', [
            'slug' => 'une-categorie-avec-des-espaces',
        ]);
    }

    #[Test]
    public function it_deletes_the_category()
    {
        $category = Category::factory()->create();

        $response = $this->get("/admin/categories/$category->id/delete");

        $response->assertRedirectToRoute('admin.categories.index');
        $this->assertDatabaseMissing('categories', [
            'id' => $category->id,
        ]);
    }

    #[Test]
    public function it_creates_category_without_highlight()
    {
        $this->post('/admin/categories', [
            'name' => 'Catégorie Simple',
            'slug' => 'categorie-simple',
            'lang' => 'fr',
            'color' => 'blue',
        ]);

        $this->assertDatabaseHas('categories', [
            'slug' => 'categorie-simple',
            'highlight' => false,
        ]);
    }
}
