<?php

namespace Tests\Feature\Controller;

use App\Http\Controllers\Admin\CreationController;
use App\Models\Category;
use App\Models\Creation;
use App\Models\UploadedPicture;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\Test;
use Tests\TestCase;

#[CoversClass(CreationController::class)]
class CreationTest extends TestCase
{
    use RefreshDatabase, WithoutMiddleware;

    #[Test]
    public function test_it_can_create_creation_with_strictly_required_fields()
    {
        Storage::fake('public');
        $category = Category::factory()->create();
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle creation',
            'category_id' => $category->id,
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => 1, 'lang' => 'fr']);

        $this->assertDatabaseHas('creations', [
            'slug' => 'nouvelle-creation',
            'is_highlighted' => false,
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Nouvelle Création',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('uploaded_pictures', [
            'id' => 1,
        ]);
    }

    #[Test]
    public function test_it_can_create_creation_with_highlighted()
    {
        Storage::fake('public');
        $category = Category::factory()->create();
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Création En Avant',
            'slug' => 'creation en avant',
            'category_id' => $category->id,
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
            'is_highlighted' => 'on', // Fourniture de 'is_highlighted' comme true
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => 1, 'lang' => 'fr']);

        $this->assertDatabaseHas('creations', [
            'slug' => 'creation-en-avant',
            'is_highlighted' => true, // Vérification que 'is_highlighted' est true
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Création En Avant',
            'locale' => 'fr',
        ]);
    }

    #[Test]
    public function test_it_can_update_creation_highlighted_status()
    {
        Storage::fake('public');
        $category = Category::factory()->create();

        $creation = Creation::factory()->create([
            'slug' => 'creation-test',
            'category_id' => $category->id,
            'is_highlighted' => false,
        ]);

        $coverImage = UploadedPicture::factory()->create();
        $creation->cover_uploaded_picture_id = $coverImage->id;
        $creation->save();

        $response = $this->put("/admin/creations/{$creation->id}", [
            'lang' => 'fr',
            'name' => 'Création Mise En Avant',
            'slug' => 'creation-mise-en-avant',
            'category_id' => $category->id,
            'is_highlighted' => 'on',
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => $creation->id, 'lang' => 'fr']);

        $this->assertDatabaseHas('creations', [
            'id' => $creation->id,
            'slug' => 'creation-mise-en-avant',
            'is_highlighted' => true,
        ]);
    }

    #[Test]
    public function test_it_fails_to_create_creation_without_name()
    {
        Storage::fake('public');
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'slug' => 'nouvelle-creation',
            'category_id' => 1,
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
        ]);

        $response->assertSessionHasErrors('name');
    }

    #[Test]
    public function test_it_fails_to_create_creation_without_category_id()
    {
        Storage::fake('public');
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
        ]);

        $response->assertSessionHasErrors('category_id');
    }

    #[Test]
    public function test_it_fails_to_create_creation_without_cover_image()
    {
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'category_id' => 1,
        ]);

        $response->assertSessionHasErrors('cover_image');
    }

    #[Test]
    public function test_it_fails_to_create_creation_with_invalid_cover_image()
    {
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'category_id' => 1,
            'cover_image' => 'invalid-image',
        ]);

        $response->assertSessionHasErrors('cover_image');
    }

    #[Test]
    public function test_it_can_create_creation_with_optional_text_fields()
    {
        Storage::fake('public');
        $category = Category::factory()->create();
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'category_id' => $category->id,
            'short_description' => 'Description courte',
            'description' => 'Description longue',
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
            'additional_images' => [
                UploadedFile::fake()->image('photo2.jpg'),
                UploadedFile::fake()->image('photo3.jpg'),
            ],
            'completion_date_start' => '2021',
            'completion_date_end' => '2022',
            'tags' => 'tag1, tag2, tag3',
            'is_highlighted' => 'on',
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => 1, 'lang' => 'fr']);

        $this->assertDatabaseHas('creations', [
            'slug' => 'nouvelle-creation',
            'category_id' => $category->id,
            'completion_date_start' => '2021',
            'completion_date_end' => '2022',
            'tags' => 'tag1, tag2, tag3',
            'is_highlighted' => true,
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Nouvelle Création',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description courte',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description longue',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('uploaded_pictures', [
            'id' => 1,
        ]);

        $this->assertDatabaseHas('uploaded_pictures', [
            'id' => 2,
        ]);

        $this->assertDatabaseHas('uploaded_pictures', [
            'id' => 3,
        ]);

        $this->assertDatabaseHas('creation_pictures', [
            'creation_id' => 1,
            'uploaded_picture_id' => 2,
        ]);

        $this->assertDatabaseHas('creation_pictures', [
            'creation_id' => 1,
            'uploaded_picture_id' => 3,
        ]);
    }

    /**
     * @throws BindingResolutionException
     */
    #[Test]
    public function test_it_can_edit_creation_text_fields()
    {
        Storage::fake('public');
        $category = Category::factory()->create();
        $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'category_id' => $category->id,
            'short_description' => 'Description courte',
            'description' => 'Description longue',
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
            'additional_images' => [
                UploadedFile::fake()->image('photo2.jpg'),
                UploadedFile::fake()->image('photo3.jpg'),
            ],
        ]);

        $createdCreation = $this->app->make('db')->table('creations')->first();
        $createdCreationNameTranslationKeyId = $createdCreation->name_translation_key_id;
        $createdCreationShortDescTranslationKeyId = $createdCreation->short_desc_translation_key_id;
        $createdCreationDescTranslationKeyId = $createdCreation->desc_translation_key_id;

        $response = $this->put("/admin/creations/{$createdCreation->id}", [
            'lang' => 'fr',
            'name' => 'Nouvelle Création Modifiée',
            'slug' => 'nouvelle-creation-modifiee',
            'category_id' => $category->id,
            'short_description' => 'Description courte modifiée',
            'description' => 'Description longue modifiée',
            'cover_image' => UploadedFile::fake()->image('photo4.jpg'),
            'additional_images' => [
                UploadedFile::fake()->image('photo5.jpg'),
                UploadedFile::fake()->image('photo6.jpg'),
            ],
            'is_highlighted' => 'on',
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => $createdCreation->id, 'lang' => 'fr']);

        $this->assertDatabaseHas('translations', [
            'text' => 'Nouvelle Création Modifiée',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description courte modifiée',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description longue modifiée',
            'locale' => 'fr',
        ]);

        $nameTranslationKeyId = $this->app->make('db')->table('translations')
            ->where('text', 'Nouvelle Création Modifiée')
            ->first()->translation_key_id;

        $shortDescriptionTranslationKeyId = $this->app->make('db')->table('translations')
            ->where('text', 'Description courte modifiée')
            ->first()->translation_key_id;

        $descriptionTranslationKeyId = $this->app->make('db')->table('translations')
            ->where('text', 'Description longue modifiée')
            ->first()->translation_key_id;

        $this->assertEquals($createdCreationNameTranslationKeyId, $nameTranslationKeyId);
        $this->assertEquals($createdCreationShortDescTranslationKeyId, $shortDescriptionTranslationKeyId);
        $this->assertEquals($createdCreationDescTranslationKeyId, $descriptionTranslationKeyId);

        $this->assertDatabaseHas('creations', [
            'id' => $createdCreation->id,
            'is_highlighted' => true, // Vérification de la mise à jour
        ]);

        $editedCreationPictures = $this->app->make('db')->table('creation_pictures')
            ->where('creation_id', $createdCreation->id)
            ->get();

        $this->assertCount(4, $editedCreationPictures);
    }

    #[Test]
    public function test_it_can_remove_additional_image_from_creation()
    {
        Storage::fake('public');
        $category = Category::factory()->create();

        $coverImage = UploadedPicture::factory()->create();

        $additionalImage1 = UploadedPicture::factory()->create();
        $additionalImage2 = UploadedPicture::factory()->create();

        $creation = Creation::factory()->create([
            'category_id' => $category->id,
            'cover_uploaded_picture_id' => $coverImage->id,
        ]);

        $creation->creationPictures()->attach([$additionalImage1->id, $additionalImage2->id]);

        $this->assertDatabaseHas('creation_pictures', [
            'creation_id' => $creation->id,
            'uploaded_picture_id' => $additionalImage1->id,
        ]);

        $this->assertDatabaseHas('creation_pictures', [
            'creation_id' => $creation->id,
            'uploaded_picture_id' => $additionalImage2->id,
        ]);

        $response = $this->delete(route('admin.creations.remove-additionnal-image', [
            'creation' => $creation->id,
            'image' => $additionalImage1->id,
        ]));

        $response->assertJson([
            'success' => true,
        ]);

        $this->assertDatabaseMissing('creation_pictures', [
            'creation_id' => $creation->id,
            'uploaded_picture_id' => $additionalImage1->id,
        ]);

        $this->assertDatabaseHas('creation_pictures', [
            'creation_id' => $creation->id,
            'uploaded_picture_id' => $additionalImage2->id,
        ]);
    }

    /**
     * @throws BindingResolutionException
     */
    #[Test]
    public function test_it_can_add_english_translation_to_the_creation()
    {
        Storage::fake('public');
        $category = Category::factory()->create();
        $response = $this->post('/admin/creations', [
            'lang' => 'fr',
            'name' => 'Nouvelle Création',
            'slug' => 'nouvelle-creation',
            'short_description' => 'Description courte',
            'description' => 'Description longue',
            'category_id' => $category->id,
            'cover_image' => UploadedFile::fake()->image('photo1.jpg'),
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => 1, 'lang' => 'fr']);

        $this->assertDatabaseHas('translations', [
            'text' => 'Nouvelle Création',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description courte',
            'locale' => 'fr',
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Description longue',
            'locale' => 'fr',
        ]);

        $createdCreation = Creation::where('slug', 'nouvelle-creation')->first();
        $createdCreationNameTranslationKeyId = $createdCreation->name_translation_key_id;
        $createdCreationShortDescTranslationKeyId = $createdCreation->short_desc_translation_key_id;
        $createdCreationDescTranslationKeyId = $createdCreation->desc_translation_key_id;

        $response = $this->put("/admin/creations/{$createdCreation->id}", [
            'lang' => 'en',
            'name' => 'New Creation',
            'slug' => 'new-creation',
            'short_description' => 'Short description',
            'description' => 'Long description',
            'category_id' => $category->id,
        ]);

        $response->assertRedirectToRoute('admin.creations.edit', ['creation' => 1, 'lang' => 'en']);

        $this->assertDatabaseHas('translations', [
            'text' => 'New Creation',
            'locale' => 'en',
            'translation_key_id' => $createdCreationNameTranslationKeyId,
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Short description',
            'locale' => 'en',
            'translation_key_id' => $createdCreationShortDescTranslationKeyId,
        ]);

        $this->assertDatabaseHas('translations', [
            'text' => 'Long description',
            'locale' => 'en',
            'translation_key_id' => $createdCreationDescTranslationKeyId,
        ]);
    }
}
