<?php

namespace Tests\Feature\Controller;

use App\Http\Controllers\Admin\LegalMentionController;
use App\Models\LegalMention;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use PHPUnit\Framework\Attributes\CoversClass;
use Tests\TestCase;

#[CoversClass(LegalMentionController::class)]
class LegalMentionTest extends TestCase
{
    use RefreshDatabase, WithoutMiddleware;

    public function test_it_can_list_legal_mentions()
    {
        LegalMention::factory()->count(3)->create();

        $response = $this->get(route('admin.legal_mentions.index'));

        $response->assertStatus(200);
        $response->assertViewIs('admin.legal_mentions.index');
        $response->assertViewHas('mentions');
        $this->assertCount(3, $response->viewData('mentions'));
    }

    public function test_it_can_create_a_legal_mention()
    {
        $data = [
            'lang' => 'fr',
            'content' => 'Contenu des mentions légales',
            'active' => 'on',
        ];

        $response = $this->post(route('admin.legal_mentions.store'), $data);

        $response->assertRedirect(route('admin.legal_mentions.index'));
        $this->assertDatabaseHas('translations', ['text' => 'Contenu des mentions légales', 'locale' => 'fr']);
        $this->assertDatabaseHas('legal_mentions', ['active' => true]);
    }

    public function test_it_fails_to_create_a_legal_mention_without_content()
    {
        $response = $this->post(route('admin.legal_mentions.store'), [
            'lang' => 'fr',
        ]);

        $response->assertSessionHasErrors('content');
    }

    public function test_it_can_edit_a_legal_mention()
    {
        $mention = LegalMention::factory()->create();

        $response = $this->get(route('admin.legal_mentions.edit', ['id' => $mention->id]));

        $response->assertStatus(200);
        $response->assertViewIs('admin.legal_mentions.edit');
        $response->assertViewHas('mention', $mention);
    }

    public function test_it_can_update_a_legal_mention()
    {
        $mention = LegalMention::factory()->create();

        $data = [
            'lang' => 'fr',
            'content' => 'Nouveau contenu',
            'active' => 'on',
        ];

        $response = $this->put(route('admin.legal_mentions.update', ['id' => $mention->id]), $data);

        $response->assertRedirect(route('admin.legal_mentions.edit', ['id' => $mention->id, 'lang' => 'fr']));
        $this->assertDatabaseHas('translations', ['text' => 'Nouveau contenu', 'locale' => 'fr']);
        $this->assertDatabaseHas('legal_mentions', ['id' => $mention->id, 'active' => true]);
    }

    public function test_it_can_delete_a_legal_mention()
    {
        $mention = LegalMention::factory()->create();

        $response = $this->get(route('admin.legal_mentions.delete', ['id' => $mention->id]));

        $response->assertRedirect(route('admin.legal_mentions.index'));
        $this->assertDatabaseMissing('legal_mentions', ['id' => $mention->id]);
    }

    public function test_it_cannot_delete_a_nonexistent_legal_mention()
    {
        $response = $this->get(route('admin.legal_mentions.delete', ['id' => 999]));

        $response->assertStatus(404);
    }

    public function test_it_can_delete_an_active_legal_mention()
    {
        $mention = LegalMention::factory()->create(['active' => true]);

        $response = $this->get(route('admin.legal_mentions.delete', ['id' => $mention->id]));

        $response->assertRedirect(route('admin.legal_mentions.index'));
        $this->assertDatabaseMissing('legal_mentions', ['id' => $mention->id]);
    }
}
