<?php

namespace Controller;

use App\Http\Controllers\Admin\PrestationController;
use App\Models\Category;
use App\Models\Prestation;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithoutMiddleware;
use Illuminate\Http\UploadedFile;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\Test;
use Tests\TestCase;

#[CoversClass(PrestationController::class)] class PrestationControllerTest extends TestCase
{
    use RefreshDatabase, WithoutMiddleware;

    #[Test]
    public function it_can_list_all_prestations()
    {
        $prestations = Prestation::factory(5)->create();

        $response = $this->get(route('admin.prestations.index'));

        $response->assertStatus(200);
        $response->assertViewIs('admin.prestations.index');
        $response->assertViewHas('prestations', function ($viewPrestations) use ($prestations) {
            return $viewPrestations->count() === $prestations->count();
        });
    }

    #[Test]
    public function it_can_display_the_create_prestation_page()
    {
        $categories = Category::factory(3)->create();

        $response = $this->get(route('admin.prestations.create'));

        $response->assertStatus(200);
        $response->assertViewIs('admin.prestations.edit');
        $response->assertViewHas('categories', function ($viewCategories) use ($categories) {
            return $viewCategories->count() === $categories->count();
        });
    }

    #[Test]
    public function it_can_store_a_new_prestation()
    {
        $category = Category::factory()->create();
        $data = [
            'lang' => 'fr',
            'name' => 'Test Prestation',
            'slug' => 'test-prestation',
            'category_id' => $category->id,
            'icon_svg' => UploadedFile::fake()->createWithContent('icon.svg', '<svg></svg>'),
            'attract_desc' => 'Attractive description',
            'content_desc' => 'Content description',
            'detail' => 'Details about the prestation',
        ];

        $response = $this->post(route('admin.prestations.store'), $data);

        $response->assertRedirect(route('admin.prestations.edit', ['prestation' => 1, 'lang' => 'fr']));
        $response->assertSessionHas('success', 'Prestation créée avec succès.');

        $this->assertDatabaseHas('prestations', [
            'category_id' => $category->id,
            'detail' => 'Details about the prestation',
        ]);
    }

    #[Test]
    public function it_can_display_the_edit_prestation_page()
    {
        $category = Category::factory()->create();
        $prestation = Prestation::factory()->create(['category_id' => $category->id]);

        $response = $this->get(route('admin.prestations.edit', ['prestation' => $prestation->id, 'lang' => 'fr']));

        $response->assertStatus(200);
        $response->assertViewIs('admin.prestations.edit');
        $response->assertViewHas('prestation', $prestation);
    }

    #[Test]
    public function it_can_update_a_prestation()
    {
        $category = Category::factory()->create();
        $prestation = Prestation::factory()->create(['category_id' => $category->id]);

        $data = [
            'lang' => 'fr',
            'name' => 'Updated Prestation',
            'slug' => 'updated-prestation',
            'category_id' => $category->id,
            'icon_svg' => UploadedFile::fake()->createWithContent('icon.svg', '<svg></svg>'),
            'attract_desc' => 'Updated description',
            'content_desc' => 'Updated content',
            'detail' => 'Updated details',
        ];

        $response = $this->put(route('admin.prestations.update', ['prestation' => $prestation->id]), $data);

        $response->assertRedirect(route('admin.prestations.edit', ['prestation' => $prestation->id, 'lang' => 'fr']));
        $response->assertSessionHas('success', 'Prestation mise à jour avec succès.');

        $this->assertDatabaseHas('prestations', [
            'id' => $prestation->id,
            'detail' => 'Updated details',
            'slug' => 'updated-prestation',
        ]);
    }

    #[Test]
    public function it_can_delete_a_prestation()
    {
        $prestation = Prestation::factory()->create();

        $response = $this->delete(route('admin.prestations.delete', ['prestation' => $prestation->id]));

        $response->assertRedirect(route('admin.prestations.index'));
        $response->assertSessionHas('success', 'Prestation supprimée avec succès.');

        $this->assertDatabaseMissing('prestations', [
            'id' => $prestation->id,
        ]);
    }
}
