<?php

namespace Tests\Feature\Controller;

use App\Http\Controllers\Admin\SettingsController;
use App\Mail\AdminInvitationMail;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use PHPUnit\Framework\Attributes\CoversClass;
use Tests\TestCase;

#[CoversClass(SettingsController::class)] class SettingsControllerTest extends TestCase
{
    use RefreshDatabase;

    public function test_it_can_create_a_user_and_send_invitation_email()
    {
        config(['app.protected_account_email' => 'test@example.com']);
        $fakedUser = User::factory()->create([
            'email' => 'test@example.com',
        ]);

        $this->actingAs($fakedUser);

        Mail::fake();

        $data = [
            'name' => 'John Doe',
            'email' => 'john.doe@example.com',
        ];

        $response = $this->post(route('admin.settings.create-user'), $data);

        $response->assertRedirect(route('admin.settings.users'));
        $response->assertSessionHas('success', 'Utilisateur créé.');

        $this->assertDatabaseHas('users', [
            'name' => 'John Doe',
            'email' => 'john.doe@example.com',
        ]);

        Mail::assertSent(AdminInvitationMail::class, function ($mail) use ($data) {
            return $mail->hasTo($data['email']);
        });

        $user = User::where('email', $data['email'])->first();
        $this->assertTrue(Hash::check(Str::random(), $user->password) === false);
    }

    public function test_it_fails_to_create_user_with_existing_email()
    {
        config(['app.protected_account_email' => 'test@example.com']);
        $fakedUser = User::factory()->create([
            'email' => 'test@example.com',
        ]);

        $this->actingAs($fakedUser);

        User::factory()->create([
            'email' => 'john.doe@example.com',
        ]);

        $data = [
            'name' => 'Jane Doe',
            'email' => 'john.doe@example.com',
        ];

        $response = $this->post(route('admin.settings.create-user'), $data);

        $response->assertSessionHasErrors('email');
    }
}
