<?php

namespace Tests\Feature\Service;

use App\Jobs\UploadedPictureJob;
use App\Models\UploadedPicture;
use App\Services\UploadedFilesService;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Queue;
use Illuminate\Support\Facades\Storage;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\Test;
use Tests\TestCase;

#[CoversClass(UploadedFilesService::class)] class UploadedFilesServiceTest extends TestCase
{
    use RefreshDatabase;

    #[Test]
    public function test_it_generates_cache_key_correctly()
    {
        $service = new UploadedFilesService;
        $path = 'images/photos';
        $disk = 'public';
        $expectedCacheKey = 'storage_folders.'.$disk.'.images-photos';

        $cacheKey = $service->pathCacheKey($path, $disk);

        $this->assertEquals($expectedCacheKey, $cacheKey, 'The cache key is not generated correctly.');
    }

    #[Test]
    public function test_it_refreshes_file_list_cache_and_returns_file_names()
    {
        Storage::fake('public');
        $service = new UploadedFilesService;
        $path = 'images/photos';
        $disk = 'public';

        Storage::disk($disk)->put($path.'/file1.jpg', 'content');
        Storage::disk($disk)->put($path.'/file2.jpg', 'content');

        $files = $service->refreshFileListCache($path, $disk);

        $this->assertEquals(['file1.jpg', 'file2.jpg'], $files);
    }

    #[Test]
    public function test_it_clears_existing_cache_when_refreshing_file_list()
    {
        Storage::fake('public');
        $service = new UploadedFilesService;
        $path = 'images/photos';
        $disk = 'public';
        $cacheKey = $service->pathCacheKey($path, $disk);

        Cache::put($cacheKey, ['old_file.jpg']);

        Storage::disk($disk)->put($path.'/file1.jpg', 'content');

        $service->refreshFileListCache($path, $disk);

        $this->assertEquals(['file1.jpg'], Cache::get($cacheKey));
    }

    #[Test]
    public function test_it_stores_and_optimizes_an_uploaded_picture()
    {
        Storage::fake('public');
        Queue::fake();

        $uploadedFile = UploadedFile::fake()->image('test.jpg');

        $service = app(UploadedFilesService::class);
        $uploadedPicture = $service->storeAndOptimizeUploadedPicture($uploadedFile);

        Storage::disk('public')->assertExists($uploadedPicture->path_original);

        Queue::assertPushed(UploadedPictureJob::class);
    }

    #[Test]
    public function test_it_throws_exception_for_unsupported_format()
    {
        $this->expectException(\Exception::class);
        $this->expectExceptionMessage('The specified file is not a picture or is not supported');

        $uploadedFile = UploadedFile::fake()->create('test.txt');

        $service = app(UploadedFilesService::class);
        $service->storeAndOptimizeUploadedPicture($uploadedFile);
    }

    #[Test]
    public function test_it_deletes_optimized_images()
    {
        Storage::fake('public');

        $uploadedPicture = UploadedPicture::factory()->create([
            'path_thumbnail' => 'uploads/test_thumbnail.avif',
            'path_medium' => 'uploads/test_medium.avif',
            'path_big' => 'uploads/test_big.avif',
            'path_fullsize' => 'uploads/test_fullsize.avif',
        ]);

        Storage::disk('public')->put($uploadedPicture->path_thumbnail, 'content');
        Storage::disk('public')->put($uploadedPicture->path_medium, 'content');
        Storage::disk('public')->put($uploadedPicture->path_big, 'content');
        Storage::disk('public')->put($uploadedPicture->path_fullsize, 'content');

        $uploadedPicture->deleteOptimized();

        Storage::disk('public')->assertMissing($uploadedPicture->path_thumbnail);
        Storage::disk('public')->assertMissing($uploadedPicture->path_medium);
        Storage::disk('public')->assertMissing($uploadedPicture->path_big);
        Storage::disk('public')->assertMissing($uploadedPicture->path_fullsize);
    }
}
